'''
Class and functions to process the point clouds



'''

import pdal, json, glob


class pcl_process(object):


	def __init__(self, project_path, ext='ply'):

		if not os.path.exists(project_path):
            print("ERROR The path " + project_path + " doesn't exists")
            return
        else:
        	self.project_path = project_path
        	os.chdir(self.project_path)

        # parameters for point cloud filtering 
        self.x_offset = 410000      # these values are default for Finse, NORWAY
		self.y_offset = 6710000     # these values are default for Finse, NORWAY

		# Cropping area
		self.crop_xmin = 416100
		self.crop_xmax = 416900
		self.crop_ymin = 6715900
		self.crop_ymax = 6716700

		# Raster bounding box, default is same as cropping box.
		self.raster_xmin = self.crop_xmin
		self.raster_xmax = self.crop_xmax
		self.raster_ymin = self.crop_ymin
		self.raster_ymax = self.crop_ymax

        # parameters for conversion to GeoTiff
        self.resolution = 1
        self.radius = self.resolution * 1.4
        self.nodata = -9999
        self.gdaldriver = "GTiff"
        self.output_type = ["min", "max", "mean", "idw", "count", "stdev"]


    def add_ply_pcl(self)
        os.chdir(self.project_path)
        self.ply_pcl_flist = glob.glob("*.ply")
        print("=======================\n PLY point clouds added: ")
        for file in self.ply_pcl_flist:
        	print(file)
        print(".......................")
        print(str(self.ply_pcl_flist.__len__()) + " point clouds added")
        print("=======================")


    def add_las_pcl(self)
        os.chdir(self.project_path)
        self.las_pcl_flist = glob.glob("*.las")
        print("=======================\n LAS point clouds added: ")
        for file in self.las_pcl_flist:
        	print(file)
        print(".......................")
        print(str(self.las_pcl_flist.__len__()) + " point clouds added")
        print("=======================")


	def pipeline_realization(pip_json, print_result):
	    try:
	        # ===============================================
	        # Pipeline execution
	        pipeline = pdal.Pipeline(pip_json)
	        pipeline.validate()  # check if our JSON and options were good

	        pipeline.execute()

	        if print_result:
	            arrays = pipeline.arrays
	            metadata = pipeline.metadata
	            log = pipeline.log
	            print("\n================")
	            print("Arrays:")
	            print(arrays)
	            print("\n================")
	            print("Metadata:")
	            print(metadata)
	            print("\n================")
	            print("Log:")
	            print(log)

	        print("pdal pipeline finished")
	        return True
	    except:
	        print(" Error !!")
	        return False


    def filter_pcl(self, file_input, file_output, print_result=True):
    	'''
    	Function to filter a point cloud: cropping to ROI, removing statistically outliers, saving output to .las format
    	'''
		pip_filter_json = json.dumps(
			{
			  	"pipeline": 
			  	[
				    file_input,
				    {
					    "type":"filters.python",
						"script":"pdal_python_filter.py",
						"function":"add_XY_UTM",
						"pdalargs":{"x_offset":self.x_offset,"y_offset":self.y_offset}
				    },
				    {
				        "type":"filters.crop",
				        "bounds":str(([self.crop_xmin, self.crop_xmax], [self.crop_ymin, self.crop_ymax]))
				    },
				    {
		                "type": "filters.range",
		                "limits": "Z[" + str(zmin) + ":" + str(zmax) + "]"
            		},
            		{
		              "type":"filters.lof",
		              "minpts":20
		            },
		            {
		              "type":"filters.range",
		              "limits":"LocalOutlierFactor[:1.2]"
		            },
					{
					    "type": "filters.range",
					    "limits": "Classification![7:12]"
					},
					{
						"type":"writers.las",
						"filename":file_ouput,
						"scale_x":1,
						"scale_y":1,
						"scale_z":1

					}
			  	]
			}
		)
		pipeline_realization(pip_filter_json, print_result=print_result)


	def filter_all_pcl(self, print_result=True):
		'''
		Function to process all pcl with filter_pcl() function
		'''
		print("=======================")
		for file in self.ply_pcl_flist:
			filter_pcl(self, file, file[:-4] + '_clean.las', print_result=print_result)
		print(".......................")
		print("All PLY files filtered")
		print("=======================")


    def convert_pcl2dem(self, input_file, output_file, print_result=True):
    	'''
    	Function to convert .las point cloud to a raster (.tif)
    	'''
    	pip_dem = json.dumps(
	    {
	        "pipeline":[
	            {"type": "readers.las",
	             "filename": file_input
	            },
	            {
	                "filename": file_ouput # file.split('.')[0] + '_' + str(resolution) + 'm.tif',
	                "gdaldriver":"GTiff",
	                "output_type":"all",
	                "resolution":self.resolution,
	                "radius": self.radius,
	                "bounds": str(([self.raster_xmin, self.raster_xmax], [self.raster_ymin, self.raster_ymax])),
	                "type": "writers.gdal",
	                "nodata":self.nodata
	            }
	        ]
	    })
	    pipeline_realization(pip_dem, print_result=print_result)


	def convert_all_pcl2dem(self, print_result=True):
		'''
		Function to process all pcl with filter_pcl() function
		'''
		print("=======================")
		for file in self.las_pcl_flist:
			convert_pcl2dem(self, file, file[:-4] + '_' + str(self.resolution) + 'm.tif', print_result=print_result)
		print(".......................")
		print("All LAS converted to DEMs")
		print("=======================")

	def extract_ortho_value(self, input_file, output_file, print_result=True):
    	'''
    	Function to convert .las point cloud to a raster (.tif)
    	'''
    	pip_ortho = json.dumps(
	    {
	        "pipeline":[
	            {"type": "readers.las",
	             "filename": input_file
	            },
	            {
	                "type": "filters.python",
	                "script": "pdal_python_filter.py",
	                "function": "rgb2value",
	                "add_dimension": "Value",
	                "module": "anything"
	            },
	            {
	                "filename": output_file#file.split('.')[0] + '_' + str(resolution) + 'm_value.tif',
	                "gdaldriver":"GTiff",
	                "output_type":"mean",
	                "dimension" : "Value",
	                "resolution":self.resolution,
	                "radius": self.radius,
	                "bounds": str(([self.raster_xmin, self.raster_xmax], [self.raster_ymin, self.raster_ymax])),
	                "type": "writers.gdal",
	                "nodata":self.nodata
	            }
	        ]
	    })
	    pipeline_realization(pip_dem, print_result=print_result)

	def extract_all_ortho_value(self, print_result=True):
		'''
		Function to process all pcl and derive an orthophoto containing the Value (computed from RGB to HSV)
		'''

		print("=======================")
		for file in self.las_pcl_flist:
			extract_ortho_value(self, file, file[:-4] + '_' + str(self.resolution) + 'm_value.tif', print_result=print_result)
		print(".......................")
		print("All LAS converted to Value orthophoto (monochrome)")
		print("=======================")


	def custom_pipeline(self, json_pipeline):
		'''
		Function to enter a custom made pdal pipeline. Input should be a Json format pipeline following the format compatible with PDAL instructions
		'''
		return json.dumps(json_pipeline)


	def apply_custom_pipeline(self, pipeline, file_list=self.las_pcl_flist,  print_result=True):
		"""
		Function to apply a custom pipeline to 
		"""
		print("=======================")
		for file in file_list:
			pipeline_realization(pipeline, print_result)
		print(".......................")
		print("Custom pipeline applied to all files")
		print("=======================")



if __name__ == "__main__":

	# Create a pcl_class object, indication the path to the photo4d project
	my_pcl = pcl_process(project_path="path_to_project_folder")

	my_pcl.resolution = 1  # set the resolution of the final DEMs

	# Set the bounding box the Region of Interest (ROI)
	my_pcl.crop_xmin = 416100
	my_pcl.crop_xmax = 416900
	my_pcl.crop_ymin = 6715900
	my_pcl.crop_ymax = 6716700
	my_pcl.nodata = -9999

	# add path og the .ply point cloud files to the python class
	my_pcl.add_ply_pcl()

	# filter the point clouds with pdal routine, and save resulting point clouds as .las file
	my_pcl.filter_all_pcl()

	# add path of the .las files
	my_pcl.add_las_pcl()

	# conver the .las point clouds to DEMs (geotiff)
	my_pcl.convert_all_pcl2dem()

	# Extract Value orthophoto from RGB 
	my_pcl.extract_all_ortho_value()

	###########
	# Custom processing pdal pipeline






'''
Custom python filters for pdal

'''
import numpy as np
import matplotlib.colors as cl
import pandas as pd
#import pdb

def add_XY_UTM(ins, outs):
	X = ins['X'] 
	X += float(pdalargs['x_offset'])
	Y = ins['Y'] 
	Y += float(pdalargs['y_offset'])
	outs['Y'] = Y
	outs['X'] = X
	return True


def voxelGrid(ins, outs):

	ROI = [float(pdalargs['Xmin']),
		float(pdalargs['Xmax']),
		float(pdalargs['Ymin']),
		float(pdalargs['Ymax']),
		float(pdalargs['Zmin']),
		float(pdalargs['Zmax'])]
	leaf = float(pdalargs['leaf'])

	df = pd.DataFrame({'X' : ins['X'] ,
		'Y' : ins['Y'] ,
		'Z' : ins['Z'],
		'Red':ins['Red'],
		'Green':ins['Green'],
		'Blue':ins['Blue']})

	for i in range(0,6):
		if ROI[i]==-9999:
			if i==0:
				ROI[i] = df.iloc[:,0].min()
			elif i==1:
				ROI[i] = df.iloc[:,0].max()
			elif i==2:
				ROI[i] = df.iloc[:,1].min()
			elif i==3:
				ROI[i] = df.iloc[:,1].max()
			elif i==4:
				ROI[i] = df.iloc[:,2].min()
			elif i==5:
				ROI[i] = df.iloc[:,2].max()
	
	#print(ROI)
	nx = np.int((ROI[1]-ROI[0])/leaf)
	ny = np.int((ROI[3]-ROI[2])/leaf)
	nz = np.int((ROI[5]-ROI[4])/leaf)

	bins_x = np.linspace(ROI[0], ROI[1], nx+1)
	df['x_cuts'] = pd.cut(df.X,bins_x, labels=False)
	bins_y = np.linspace(ROI[2],ROI[3], ny+1)
	df['y_cuts'] = pd.cut(df.Y,bins_y, labels=False)
	bins_z = np.linspace(ROI[4],ROI[5], nz+1)
	df['z_cuts'] = pd.cut(df.Z,bins_z, labels=False)

	grouped = df.groupby([df['x_cuts'],df['y_cuts'], df['z_cuts']])

	outf = pd.DataFrame()
	outf['X'] = np.hstack((grouped.X.mean().reset_index().X, np.zeros(ins['X'].shape[0] - grouped.X.mean().reset_index().X.shape[0])-9999))
	outf['Y'] = np.hstack((grouped.Y.mean().reset_index().Y, np.zeros(ins['X'].shape[0] - grouped.X.mean().reset_index().X.shape[0])-9999))
	outf['Z'] = np.hstack((grouped.Z.mean().reset_index().Z, np.zeros(ins['X'].shape[0] - grouped.X.mean().reset_index().X.shape[0])-9999))
	outf['Red'] = np.hstack((grouped.Red.mean().reset_index().Red, np.zeros(ins['X'].shape[0] - grouped.X.mean().reset_index().X.shape[0])-9999))
	outf['Green'] = np.hstack((grouped.Green.mean().reset_index().Green, np.zeros(ins['X'].shape[0] - grouped.X.mean().reset_index().X.shape[0])-9999))
	outf['Blue'] = np.hstack((grouped.Blue.mean().reset_index().Blue, np.zeros(ins['X'].shape[0] - grouped.X.mean().reset_index().X.shape[0])-9999))
	outf['Classification'] = (outf.X==-9999)*13
	outf = outf.dropna()

	outs['X'] = np.array(outf.X.astype('<f8'))
	outs['Y'] = np.array(outf.Y.astype('<f8'))
	outs['Z'] = np.array(outf.Z.astype('<f8'))
	outs['Red'] = np.array(outf.Red.astype('u2'))
	outs['Green'] = np.array(outf.Green.astype('u2'))
	outs['Blue'] = np.array(outf.Blue.astype('u2'))
	outs['Classification'] = np.array(outf.Classification.astype('u1'))

	return True


def reject_undefined_cloud(ins, outs):
	"""
	Filter to cut points below a certain threshold, and where there are two clouds within a cell exceeding a threshold distance
	
	Classifies points to reject with the value 10
	"""
	thresh = float(pdalargs['thresh'])
	Zmin = float(pdalargs['Zmin'])
	xstart = float(pdalargs['xstart'])-1
	xend = float(pdalargs['xend'])+1
	ystart = float(pdalargs['ystart'])-1
	yend = float(pdalargs['yend'])+1
	nx = float(pdalargs['nx'])+2
	ny = float(pdalargs['ny'])+2

	print("Cell size in X = " + str((xend-xstart)/nx))
	print("Cell size in Y = " + str((yend-ystart)/ny))
	
	df = pd.DataFrame({'X' : ins['X'] , 'Y' : ins['Y'] , 'Z' : ins['Z']})
	bins_x = np.linspace(xstart, xend, nx+1)
	df['x_cuts'] = pd.cut(df.X,bins_x, labels=False)
	bins_y = np.linspace(ystart,yend, ny+1)
	df['y_cuts'] = pd.cut(df.Y,bins_y, labels=False)

	grouped = df.groupby([df['x_cuts'],df['y_cuts']])
	ext = grouped.Z.max() - grouped.Z.min()
	ext = np.array(ext.unstack())

	df.range = ext[df.x_cuts.astype(int)-1, df.y_cuts.astype(int)-1]
	cls = ins['Classification']
	out = np.array((df.range>thresh) | (df.Z<Zmin))
	cls[out] = 12
	
	outs['Classification'] = cls.astype('u1')
	
	return True


def classify_SnowGround(ins, outs):
	"Classify points Snow/Ground based on point Value in the HSV colorspace, 1 for snow, 3 for ground"
	thresh = 180

	cls = ins['Classification']
	rgb = np.vstack((ins['Red'], ins['Green'], ins['Blue'])).T
	hsv = cl.rgb_to_hsv(rgb)
	snow = (hsv[:,2] > thresh) * 3
	ground = (hsv[:,2] <= thresh) * 2

	dt = np.dtype(('u1'))
	snow = snow.astype(dt)
	ground = ground.astype(dt)
	cls =  snow + ground

	outs['Classification'] = cls

	return True


def mask_ground(ins, outs):
	"Mask to keep ground patches only based on point Value in the HSV colorspace"
	thresh = 180

	rgb = np.vstack((ins['Red'], ins['Green'], ins['Blue'])).T
	hsv = cl.rgb_to_hsv(rgb)
	ground = hsv[:,2] <= thresh
	outs['Mask'] = ground

	return True


def rgb2value(ins, outs):
	rgb = np.vstack((ins.get('Red'), ins.get('Green'), ins.get('Blue'))).T
	hsv = cl.rgb_to_hsv(rgb)
	#pdb.set_trace()
	outs['Value'] = hsv[:,2].astype('float64')

	return True



def mask_snow(ins, outs):
	"Mask to keep ground patches only based on point Value in the HSV colorspace"
	thresh = 180

	rgb = np.vstack((ins['Red'], ins['Green'], ins['Blue'])).T
	hsv = cl.rgb_to_hsv(rgb)
	snow = hsv[:,2] > thresh
	outs['Mask'] = snow

	return True



# coding : uft8
# Generic imports
import os
from shutil import rmtree, move
# Photo4D imports
from photo4d.Utils import exec_mm3d
import re


def process_one_timestep(work_folder, pictures_array, timestep, output_folder,
                          clahe=False, tileGridSize_clahe=8, zoomF=1,
                          master_folder_id=0, Ori='Bascule', DefCor=0.0,
                          shift=None, keep_rasters=True, display_micmac=False):
    os.chdir(work_folder)
    I, J = pictures_array.shape
    
    if pictures_array[timestep, 1]: #just skip if the set is invalid somehow
            # Setup parameters
            selected_line=pictures_array[timestep]
            img_set="("
            for j in range(2,J):
                img_set += selected_line[j] + "|"
            img_set = img_set[:-1] + ")"
                
            master_img=selected_line[master_folder_id + 2]
            date_str=selected_line[0]
            ply_name= date_str + '.ply'
            
            # Don't run again if already existing
            if os.path.exists(os.path.join(work_folder,ply_name)):
                move(os.path.join(work_folder,ply_name), os.path.join(output_folder,ply_name))
            if not os.path.exists(os.path.join(output_folder,ply_name)):                      
                # Define Malt command and run it
                command='mm3d Malt GeomImage {} {} Master={} DefCor={} ZoomF={}'.format(img_set, Ori,
                                                                                     master_img, DefCor, zoomF)
                print(command)
                success, error = exec_mm3d(command, display_micmac)
                
    
                # Find the last depth map and correlation file 
                    # Get a list of all the files in the Malt output folder
                files=[]
                for f in os.walk(os.path.join(work_folder,'MM-Malt-Img-'+ master_img[:-4])):
                    for file in f:
                        files=file
                nuage_re = re.compile(r'NuageImProf_STD-MALT_Etape_\d{1}.xml')
                correlation_re = re.compile(r'Correl_STD-MALT_Num_\d{1}.tif')
                depth_re = re.compile(r'Z_Num\d{1}_DeZoom' + str(zoomF) +'_STD-MALT.tif')
                nuage_files = [ x for x in files if nuage_re.match(x)]
                correlation_files = [ x for x in files if correlation_re.match(x)]
                depth_files = [ x for x in files if depth_re.match(x)]
                sorted_nuage_files = sorted(nuage_files,reverse=True)
                sorted_correlation_files = sorted(correlation_files,reverse=True)
                sorted_depth_files = sorted(depth_files,reverse=True)
                last_nuage=sorted_nuage_files[0]
                last_cor=sorted_correlation_files[0]
                last_depth=sorted_depth_files[0]
                
                # Create the point cloud            
                if shift is None:
                    command = 'mm3d Nuage2Ply MM-Malt-Img-{}/{} Attr={} Out={}'.format(
                        '.'.join(master_img.split('.')[:-1]), last_nuage, master_img, ply_name)
                else:
                    command = 'mm3d Nuage2Ply MM-Malt-Img-{}/{} Attr={} Out={} Offs={}'.format(
                        '.'.join(master_img.split('.')[:-1]), last_nuage, master_img, ply_name, str(shift).replace(" ", ""))
            
                print(command)
                success, error = exec_mm3d(command, True)
                
                # Copy result to result folder
                # .ply point cloud
                move(os.path.join(work_folder,ply_name), os.path.join(output_folder,ply_name))
                # If we want to keep the correlation map and the depth map
                if(keep_rasters):
                    move(os.path.join(work_folder,'MM-Malt-Img-' + master_img,last_cor), os.path.join(output_folder,date_str + '_Correlation.tif'))
                    move(os.path.join(work_folder,'MM-Malt-Img-' + master_img,last_depth), os.path.join(output_folder,date_str + '_DepthMap.tif'))
                
                # Clean-up
                
                try:
                    rmtree(os.path.join(work_folder,'MM-Malt-Img-' + master_img[:-4]))
                except FileNotFoundError:
                    pass
                except PermissionError:
                    print("Permission Denied, cannot delete " + os.path.join(work_folder,'MM-Malt-Img-' + master_img))
                except OSError:
                    pass
                try:
                    rmtree(os.path.join(work_folder,"Pyram"))
                except PermissionError:
                    print("Permission Denied, cannot delete Pyram folder")
                except OSError:
                    pass
    # Go back to project folder    
    os.chdir('../') 
                

def process_all_timesteps(work_folder, pictures_array, output_folder,
                          clahe=False, tileGridSize_clahe=8, zoomF=1,
                          master_folder_id=0, Ori='Bascule', DefCor=0.0,
                          shift=None, keep_rasters=True, display_micmac=False):
    """
    Run MicMac (mm3d Malt) on all valid picture sets
    It is advised to give only absolute path in parameters


    :param work_folder: folder where the images and orientations are
    :param pictures_array: array with set definitions (also, validity of sets and timestamp)
    :param output_folder: directory for saving results
    :param clahe: if True, apply a "contrast limited adaptive histogram equalization" on the pictures before processing
    
    MicMac parameters: (see more documentation on official github and wiki (hopefully))
    :param zoomF: final zoom in the pyramidal correlation scheme
    :param master_folder_id: id of the folder containing the master images (central image of sets)
    :param Ori: Orientation to use for the correlation (Def='Bascule', the output of 'Class_photo4D.compute_transform(True)')
    :param DefCor: correlation threshold to reject area in the correlation process ([0-1] def=0)
    :param shift: shift for saving ply (if numbers are too big for 32 bit ply) [shiftE, shiftN, shiftZ]
    :param keep_rasters: keep the depth map and last correlation map
    :param display_micmac: show MicMac consol output, only usefull to follow individual set correlation status
    :return:
    """
    # ==================================================================================================================
    # checking path and parameters :
    nb_folders = len(pictures_array)

    if type(master_folder_id) != int or not (0 <= master_folder_id < nb_folders):
        print("Invalid value {} for parameter master folder, value set to 0".format(master_folder_id))
        print("must be one index of the array secondary_folder_list")
        master_folder_id = 0
        
    # make output folder if not already present
    if not os.path.exists(output_folder): os.makedirs(output_folder)
    
    # Go through set for each set
    I, J = pictures_array.shape
    for timestep in range(I):
        process_one_timestep(work_folder, pictures_array, timestep, output_folder,
                          clahe=clahe, tileGridSize_clahe=tileGridSize_clahe, zoomF=zoomF,
                          master_folder_id=master_folder_id, Ori=Ori, DefCor=DefCor,
                          shift=shift, keep_rasters=keep_rasters, display_micmac=display_micmac)




# coding : utf8
"""

To keep metadata when we transform the picture, we use the module pyxif (MIT License), available at :
                         https://github.com/zenwerk/Pyxif
"""


from datetime import datetime
import pyxif, os
import cv2 as cv
import numpy as np



def sort_pictures(folder_path_list, output_path, ext="jpg", time_interval=600):
    """
    Regroup pictures from different folders if they are taken within timeInterval seconds of interval.
    Result is stored in an array/file,
    :param folder_path_list: list of the path to folders containing pictures, each folder is for one camera
    :param output_path: path of the output .txt file containing picture sets
    :param ext: extension of the pictures
    :param time_interval: interval in seconds, with corresponds to the maximum time elapsed between the shooting of pics
    :return: array with the sorted pictures. For each set a boolean is added, always True, but can be modified later
    """
    print("\n Collecting files\n..........................................")
    # create a list containing image names and dates for each folder
    list = []
    for folder_path in folder_path_list:
        image_date_list = []
        flist = os.listdir(folder_path)
        for filename in flist:
            try:
                if filename.split(".")[-1].lower() == ext.lower():
                    image_date_list.append((filename, load_date(os.path.join(folder_path,filename))))
            except IndexError:
                pass
        list.append(image_date_list)
    if len(list) < 1:
        print("WARNING not enough folder\nTwo or more folders are needed to sort files")
        return None
    elif [] in list:
        print("WARNING No image found in One or many folder(s)")
        return None

    sorted_pictures = []
    print(" Checking dates\n..........................................")
    with open(output_path, 'w') as f:
        f.write("# Pictures taken within {} of interval\n".format(time_interval))

    good, bad = 0, 0  # counters for correct and wrong sets
    # loop on the image of the first folder
    for image_ref in list[0]:
        date_ref = image_ref[1]
        pic_group = np.empty(len(list) + 2, dtype=object)
        pic_group[0] = date_ref.strftime("%Y-%m-%dT%H-%M-%S")
        pic_group[1] = False  # the pic_group[0] is a boolean, True if all a picture is found in every folder
        pic_group[2] = image_ref[0]
        # for_file = [image_ref[0]]  # list of the images taken within the interval

        # check for pictures taken whithin the interval
        for j in range(1, len(list)):
            folder = list[j]  # list of the (filename,date) of one folder
            i, found = 0, False
            while not found and i < len(folder):
                date_var = folder[i][1]
                diff = abs(date_ref - date_var)
                if diff.days * 86400 + diff.seconds < time_interval:  # if the two pictures are taken within 10 minutes
                    found = True
                    pic_group[j + 2] = folder[i][0]
                i += 1

        if None not in pic_group:
            good += 1
            pic_group[1] = True
            print(" Pictures found in every folder corresponding to the timeInterval " + pic_group[0] + "\n")
        else:
            bad += 1
            print(" Missing picture(s) corresponding to the timeInterval " + pic_group[0] + "\n")
                  
        sorted_pictures.append(pic_group)                 
        with open(output_path, 'a') as f:
            f.write(pic_group[0] + "," + str(pic_group[1]) + "," + ",".join(str(pic_group[2:])) + "\n")

    end_str = "# {} good set of pictures found, {} uncomplete sets, on a total of {} sets".format(good, bad, good + bad)
    print(end_str)
    with open(output_path, 'a') as f:
        f.write(end_str)
    return np.array(sorted_pictures)



def check_picture_quality(folder_list, output_path, pictures_array, lum_inf, blur_inf):
    """
    This function is supposed to be called after sort_pictures, as it uses the kind of array created by sort_pictures,
    which could be either collected from the return value of the function, or the file "linked_files.txt" created in
    the main folder
    It will filter pictures in which brightness is inferior to lum_inf and the "blur" (variance of Laplacian) is
    inferior to blur_min
    :param folder_list:
    :param output_path:
    :param pictures_array:
    :param lum_inf:
    :param blur_min:
    :return: same array, but some booleans will be set to False
    """
    print("\n Checking pictures\n..........................................")

    with open(output_path, 'w') as f:
        f.write(
            "# Pictures filtered with a minimum value of {} for brightness, {} for the variance of Laplacian\n".format(
                lum_inf, blur_inf))

    good, bad = 0, 0
    I, J = pictures_array.shape
    for i in range(I):
        if pictures_array[i, 1]:
            min_lum = 9999
            min_blur = 9999
            for j in range(2, J):
                path = os.path.join(folder_list[j - 2],pictures_array[i, j])
                lum = load_bright(path)

                if lum < min_lum:
                    min_lum = lum
                blur = blurr(path, 3)
                if blur < min_blur:
                    min_blur = blur

            if min_lum < lum_inf or min_blur < blur_inf:
                pictures_array[i, 1] = False
                bad += 1
            else:
                good += 1

    with open(output_path, 'a') as f:
        for line in pictures_array:
            f.write(str(line[0]) + "," + str(line[1]) + "," + ",".join(str(line[2:])) + "\n")
        end_line = " {} good set of pictures found, {} rejected sets, on a total of {} sets".format(good, bad, good + bad)
        f.write("#" + end_line)
        print(end_line)
    return pictures_array



def load_date(filename):
    """
    Load date of the shot, according to te image metadata
    :param filename: name/path of the file
    :return: datetime format
    """
    try:
        zeroth_dict, exif_dict, gps_dict = pyxif.load(filename)
        date,time=exif_dict[pyxif.PhotoGroup.DateTimeOriginal][1].split(" ")
        year, month,day = date.split(":")
        hour,minute,sec = time.split(":")
        dateimage= datetime(int(year), int(month), int(day), int(hour), int(minute) ,int(sec))
        return dateimage
    except KeyError:
        print("WARNING No date for file " + filename)
        return None
    except FileNotFoundError:
        print("WARNING Could not find file " + filename )
        return None


def load_bright(filename):
    """
    Load luminosity of the shot scene, according to te image metadata
    :param filename: name/path of the file
    :return: float, level of brightness

    TODO: Add a method to estimate BrightnessValue of image if the field is not available from picture EXIF.
    """
    try:
        zeroth_dict, exif_dict, gps_dict = pyxif.load(filename)
        num,denom=exif_dict[pyxif.PhotoGroup.BrightnessValue][1]
        brightness=num/denom
        return brightness
    except KeyError:
        print("WARNING No brightness data for file " + filename)
        print("Check if your exif data contains a 'BrightnessValue' tag ")
        return None
    except FileNotFoundError:
        print("WARNING Could not find file " + filename )
        return None


def calc_lum(filename):
    image_bgr = cv.imread(filename)
    image_lab = cv.cvtColor(image_bgr, cv.COLOR_BGR2LAB)
    average_lum = cv.mean(cv.split(image_lab)[0])
    return average_lum



def process_clahe_folder(in_folder, tileGridSize, grey=False, out_folder="", clip_limit=2,new_name_end="_Clahe"):
    """
    Apply CLAHE to all jpeg files if a given folder
    It is not possible to overwrite files, because the initial files are needed to copy-past metadata

    :param in_folder: input folder path
    :param tileGridSize: size of the "blocks" to apply local histogram equalization
    :param grey: if True, the image will be converted to grayscale
    :param clip_limit: contrast limit, used to avoid too much noise
    :param out_folder: output folder path
    :param new_name_end: string put at the end of output files, without the extension
    :return:
    """

    # Process all the jpeg pictures in the following folder
    flist = np.sort(os.listdir(in_folder))

    for f in flist:
        try:
            if f.split(".")[-1].lower() in ["jpg","jpeg"]:
                in_path = in_folder + f
                if out_folder == "":  out_folder = in_folder
                out_path = out_folder + f[:-4] + new_name_end + ".JPG"

                process_clahe(in_path, tileGridSize, grey=grey, out_path=out_path, clip_limit=clip_limit)
        except IndexError:
            pass


def process_clahe(in_path, tileGridSize, grey=False, out_path="", clip_limit=2):
    """
    Appy CLAHE (contrast limited adaptive histogram equalization) method on an image
    for more information about CLAHE, see https://docs.opencv.org/3.1.0/d5/daf/tutorial_py_histogram_equalization.html

    Overwriting image will raise an error, as the initial image is needed to copy-past metadata
    :param in_path: input image
    :param tileGridSize: size of the "blocks" to apply local histogram equalization
    :param grey: if True, the image will be converted to grayscale
    :param out_path: output path, the folders must exists and the image extension must be valid
            by default, output will be saved as input_path/input_name_clahe.JPG
    :param clip_limit: contrast limit, used to avoid too much noise
    """
    if out_path == "":
        out_path = ".".join(in_path.split(".")[:-1]) + "_clahe.JPG"

    # read input
    print("Processing CLAHE method on " + in_path.split("/")[-1])
    img = cv.imread(in_path)

    # convert color to gray
    if grey: img = cv.cvtColor(img, cv.COLOR_BGR2GRAY)

    # apply a median filter before clahe
    img = cv.medianBlur(img, 3)  
    
    # create clahe object
    clahe = cv.createCLAHE(clipLimit=clip_limit, tileGridSize=(tileGridSize, tileGridSize))  # CLAHE

    # apply CLAHE for each image channel, and then recreate the full image (only useful if gray==False)
    channels_ini = cv.split(img)
    channels_final = []
    for channel in channels_ini:
        # Apply CLAHE
        channels_final.append(clahe.apply(channel))
    img_final = cv.merge(channels_final)

    # save image and write metadata from initial file
    cv.imwrite(out_path, img_final)
    pyxif.transplant(in_path, out_path)

def blurr(filename,ksize = 3):
    image_bgr = cv.imread(filename)  # todo la converstion en gray devrait être fait à cette ligne
    # image_gray = cv.cvtColor(image_bgr, cv.COLOR_BGR2GRAY)
    return np.log(cv.Laplacian(image_bgr, cv.CV_64F,ksize=ksize).var())


''' 
Program XXX Part I



'''

# import public library
import os
from os.path import join as opj
import numpy as np
from typing import Union
from shutil import copyfile, rmtree, copytree

# Import project libary
import photo4d.Process as proc
import photo4d.Utils as utils
import photo4d.Detect_Sift as ds
import photo4d.Image_utils as iu


class Photo4d(object):
    # Class constants
    # folders
    IMAGE_FOLDER = 'Images'
    ORI_FOLDER = "Ori-Ini"
    ORI_FINAL = "Ori-Bascule"
    MASK_FOLDER = 'Masks'
    GCP_FOLDER = 'GCP'
    RESULT_FOLDER = "Results"
    # file names
    GCP_COORD_FILE_INIT = 'GCPs_coordinates.xml'
    GCP_COORD_FILE_FINAL = 'GCPs_pick-S3D.xml'
    DF_DETECT_FILE = 'df_detect.csv'
    SET_FILE = 'set_definition.txt'
    GCP_PRECISION=0.2 # GCP precision in m
    GCP_POINTING_PRECISION=10 # Pointing precision of GCPs in images (pixels)
    GCP_PICK_FILE = 'GCPs_pick.xml'
    GCP_PICK_FILE_2D = 'GCPs_pick-S2D.xml'
    GCP_DETECT_FILE = 'GCPs_detect-S2D.xml'
    GCP_NAME_FILE = 'GCPs_names.txt'
    shift=[410000, 6710000, 0]
    # Parameters
    distortion_model="Figee"
    
    
    def __init__(self, project_path, ext='JPG'):
        if not os.path.exists(project_path):
            print("ERROR The path " + project_path + " doesn't exists")
            return
            
        # add main folder
        self.project_path = os.path.abspath(project_path)
        print("Creation of object photo4d on the folder " + self.project_path)
        
        # add camera folders
        if os.path.exists(opj(self.project_path, Photo4d.IMAGE_FOLDER)):
            self.cam_folders = [opj(self.project_path, Photo4d.IMAGE_FOLDER, cam) for cam in
                                os.listdir(opj(self.project_path, Photo4d.IMAGE_FOLDER))]
            self.nb_folders = len(self.cam_folders)
            print("Added {} camera folders : \n  {}".format(self.nb_folders, '\n  '.join(self.cam_folders)))
        else:
            print('You must create a folder "' + Photo4d.IMAGE_FOLDER + '/" containing your camera folders')
            return
            
        # =========================================================================
        # add picture sets
        picture_set_def = opj(self.project_path, Photo4d.SET_FILE)
        if os.path.exists(picture_set_def):
            self.sorted_pictures = utils.pictures_array_from_file(picture_set_def)
            print("Added picture sets from " + picture_set_def)
        else:
            self.sorted_pictures = None
        # set default selected set to the last one
        self.selected_picture_set = -1
        
        # =========================================================================
        # add initial orientation
        if os.path.exists(opj(self.project_path, Photo4d.ORI_FOLDER)):
            print("Added initial orientation")
            self.in_ori = opj(self.project_path, Photo4d.ORI_FOLDER)
        else:
            self.in_ori = None
            
        # =========================================================================
        # add image masks
        if os.path.exists(opj(self.project_path, Photo4d.MASK_FOLDER)):
            self.masks = opj(self.project_path, Photo4d.MASK_FOLDER)
            print("Masks created from ")  # todo add the set of masks (and ori)
        else:
            self.masks = None
            
        # add GCP initial files
        # =========================================================================
        if os.path.exists(opj(self.project_path, Photo4d.GCP_FOLDER, Photo4d.GCP_COORD_FILE_INIT)):
            self.gcp_coord_file = opj(self.project_path,Photo4d.GCP_FOLDER, Photo4d.GCP_COORD_FILE_INIT)
            print("Added gcp coordinates file")
        else:
            self.gcp_coord_file = None
        if os.path.exists(opj(self.project_path, Photo4d.GCP_FOLDER, Photo4d.GCP_NAME_FILE)):
            self.gcp_names = opj(self.project_path, Photo4d.GCP_FOLDER, Photo4d.GCP_NAME_FILE)
        else:
            self.gcp_names = None
            
        # extension of the images
        self.ext = ext
        
        # condition on picture dates, to process only a few sets
        self.cond = None
        
        # Create temp folder
        self.tmp_path = opj(self.project_path, "tmp")
        if not os.path.exists(self.tmp_path): 
            os.makedirs(self.tmp_path)

        
        
    def __str__(self):
        string = "\n=======================================================================\n" \
                 "Project Photo4d located at " + self.project_path + \
                 "\n======================================================================="
        string += "\n Contains {} camera folders : \n   {}".format(self.nb_folders, '\n   '.join(self.cam_folders))
        if self.sorted_pictures is None:
            string += "\n Pictures unsorted"
        else:
            string += "\n Pictures sorted in {} sets ".format(len(self.sorted_pictures))
            string += "\n The current selected set is {}".format(self.sorted_pictures[self.selected_picture_set][1:])
            
        string += "\n=======================================================================\n"
        if self.in_ori is not None:
            string += " Initial orientation computed"
            string += "\n=======================================================================\n"
            
        if self.masks is not None:
            string += " Masks done "
            string += "\n=======================================================================\n"
            
        if self.gcp_coord_file is not None:
            string += " Absolute coordinates of GCPs are given"
            if self.dict_image_gcp is not None:
                string += "\n GCPs image coordinates are computed "
            string += "\n=======================================================================\n"
            
        return string
    
    def sort_picture(self, time_interval=600):
        self.sorted_pictures = iu.sort_pictures(self.cam_folders, opj(self.project_path, Photo4d.SET_FILE),
                                                  time_interval=time_interval,
                                                  ext=self.ext)
        return self.sorted_pictures


    def check_picture_quality(self, luminosity_thresh=1, blur_thresh=6):
        '''
        Function to Check if pictures are not too dark and/or too blurry (e.g. fog)
        '''
        if self.sorted_pictures is None:
            print("ERROR You must launch the sort_pictures() method before check_pictures()")
            return
        self.sorted_pictures = iu.check_picture_quality(self.cam_folders, opj(self.project_path, Photo4d.SET_FILE),
                                                   self.sorted_pictures,
                                                   lum_inf=luminosity_thresh,
                                                   blur_inf=blur_thresh)
        return self.sorted_pictures

            
    def timeSIFT_orientation(self, resolution=5000, distortion_mode='Fraser', display=False, clahe=False,
                            tileGridSize_clahe=8):
        '''
        Function to initialize camera orientation of the reference set of images using the Micmac command Tapas
        '''
        # change from working dir to tmp dir
        os.chdir(self.tmp_path)
        
        # select the set of good pictures to estimate initial orientation
        
        
        for s in range(len(self.sorted_pictures)):
            if self.sorted_pictures[s, 1]:
                selected_line = self.sorted_pictures[s]
                
                for i in range(len(self.cam_folders)):
                    in_path = opj(self.cam_folders[i], selected_line[i + 2])
                    out_path = opj(self.tmp_path, selected_line[i + 2])
                    if clahe:
                        iu.process_clahe(in_path, tileGridSize_clahe, out_path=out_path)
                    else:
                        copyfile(in_path, out_path)

        # Execute mm3d command for orientation
        success, error = utils.exec_mm3d("mm3d Tapioca All {} {}".format(".*" + self.ext, resolution), display=display)
        success, error = utils.exec_mm3d(
            "mm3d Tapas {} {} Out={}".format(distortion_mode, ".*" + self.ext, Photo4d.ORI_FOLDER[4:]), display=display)

        ori_path = opj(self.project_path, Photo4d.ORI_FOLDER)
        if success == 0:
            # copy orientation file
            if os.path.exists(ori_path): rmtree(ori_path)
            copytree(opj(self.tmp_path, Photo4d.ORI_FOLDER), ori_path)
            self.in_ori = ori_path
        else:
            print("ERROR Orientation failed\nerror : " + str(error))

        os.chdir(self.project_path)

            
    def create_mask(self, del_pictures=True):
        '''
        Note : Only the mask of the central (MASTER) image is necessary
        '''
        
        # add mkdir, and change dir
        mask_path = opj(self.project_path, Photo4d.MASK_FOLDER)
        if not os.path.exists(mask_path): os.makedirs(mask_path)
        # select the set of good pictures to estimate initial orientation
        selected_line = self.sorted_pictures[self.selected_picture_set]
        for i in range(len(self.cam_folders)):
            in_path = opj(self.cam_folders[i], selected_line[i + 1])
            out_path = opj(mask_path, selected_line[i + 1])
            copyfile(in_path, out_path)
            ds.exec_mm3d('mm3d SaisieMasqQT {}'.format(out_path))
            if del_pictures:
                os.remove(out_path)
        self.masks = mask_path
        
    def prepare_gcp_files(self, gcp_coords_file, file_format='N_X_Y_Z', display=True):
        '''
        Function to prepare GCP coordinate from a textfile to Micmac xml format. Make sure your text file format is correct
        '''

        if not os.path.exists(opj(self.project_path, Photo4d.GCP_FOLDER)):
            os.makedirs(opj(self.project_path, Photo4d.GCP_FOLDER))
            
        # copy coordinates file into the project
        path2txt = opj(self.project_path, Photo4d.GCP_FOLDER, Photo4d.GCP_COORD_FILE_INIT)[:-4] + ".txt"
        copyfile(gcp_coords_file, path2txt)
        
        success, error = utils.exec_mm3d('mm3d GCPConvert #F={} {}'.format(file_format, path2txt),
                                         display=display)
        if success == 0:
            self.gcp_coord_file = opj(self.project_path, Photo4d.GCP_FOLDER, Photo4d.GCP_COORD_FILE_INIT)
            gcp_table = np.loadtxt(path2txt, dtype=str)

            try:
                gcp_name = gcp_table[:, file_format.split('_').index("N")]
                np.savetxt(opj(self.project_path, Photo4d.GCP_FOLDER, Photo4d.GCP_NAME_FILE), gcp_name, fmt='%s',
                           newline=os.linesep)
                self.gcp_names = opj(self.project_path, Photo4d.GCP_FOLDER, Photo4d.GCP_NAME_FILE)
            except ValueError:  # todo add a coherent except
                print("ERROR prepare_GCP_files(): Check file format and file delimiter. Delimiter is any space")
        else:
            print("ERROR prepare_GCP_files(): Check file format and file delimiter. Delimiter is any space")
            return 0
            
    def pick_initial_gcps(self):
        '''
        Function to pick GCP locations on the reference set of images with no a priori.
        
        Pick few GCPs (3 to 5) that MicMac can do a rough estimate of the camera orientation. Then go to pick_gcp_basc() to pick all GCPs of known location
        '''        
        os.chdir(self.tmp_path)
        
        if self.gcp_coord_file is None or self.gcp_names is None:
            print("ERROR prepare_gcp_files must be applied first")
        gcp_path = opj(self.project_path, Photo4d.GCP_FOLDER)
        copytree(opj(gcp_path), opj(self.tmp_path))
        # select the set of image on which to pick GCPs manually
        selected_line = self.sorted_pictures[self.selected_picture_set]
        file_set = "("
        for i in range(len(self.cam_folders)):
            file_set += selected_line[i + 1] + "|"
        file_set = file_set[:-1] + ")"
            
        commandSaisieAppuisInitQt='mm3d SaisieAppuisInitQt "{}" Ini {} {}'.format(file_set, self.GCP_NAME_FILE,
                                                                   self.GCP_PICK_FILE)
        print(commandSaisieAppuisInitQt)
        utils.exec_mm3d(commandSaisieAppuisInitQt)
        
        # Go back from tmp dir to project dir        
        os.chdir(self.project_path)
        
        
    def pick_all_gcps(self, resolution=5000):
        '''
        Function to pick GCP locations on the reference set of images, with a predicted position.
        
        Pick all GCPs of known location.
        '''

        os.chdir(self.tmp_path)

        # select the set of image on which to pick GCPs manually
        selected_line = self.sorted_pictures[self.selected_picture_set]
        file_set = "("
        for i in range(len(self.cam_folders)):
            file_set += selected_line[i + 1] + "|"
        file_set = file_set[:-1] + ")"

        command='mm3d SaisieAppuisPredicQt "{}" Bascule-Ini {} {}'.format(file_set,
                                                                   self.GCP_COORD_FILE_INIT,
                                                                   self.GCP_PICK_FILE)
        print(command)        
        utils.exec_mm3d(command)

        # Go back from tmp dir to project dir        
        os.chdir(self.project_path)
        
    def compute_transform(self, doCampari=False):
        '''
        Function to apply the transformation computed from the GCPs to all images.
        
        Set doCampari=True once all points are input and you are ready to carry on.
        '''

        os.chdir(self.tmp_path)

        # select all the images
        file_set = ".*" + self.ext
        
        commandBasc = 'mm3d GCPBascule {} Ini Bascule-Ini {} {}'.format(file_set,
                                                                   self.GCP_COORD_FILE_INIT,
                                                                   self.GCP_PICK_FILE_2D)
        print(commandBasc)
        utils.exec_mm3d(commandBasc)
        
        if(doCampari):
            command = 'mm3d Campari {} Bascule-Ini Bascule GCP=[{},{},{},{}] AllFree=1'.format(file_set, self.GCP_COORD_FILE_INIT, self.GCP_PRECISION, self.GCP_PICK_FILE_2D, self.GCP_POINTING_PRECISION)
            print(command)
            utils.exec_mm3d(command)
            copytree(opj(self.tmp_path, 'Ori-Bascule'),  Photo4d.ORI_FINAL)
             
        # Go back from tmp dir to project dir        
        os.chdir(self.project_path)

            
    def pick_ManualTiePoints(self):
        '''
        Function to pick additional points that can be set as 'GCPs'. These will get coordinates estimates based on camera orientation, and will be used in other set of images for triangulation.
        This way, we artificailly increase the number of GCPs, and use the selected set of reference images as the absolute reference to which other 3D model will be orientated against. 
        
        Pick as many points as possible that are landmarks across the all set of image.
        '''

        os.chdir(self.tmp_path)

        # select the set of image on which to pick GCPs manually
        selected_line = self.sorted_pictures[self.selected_picture_set]
        file_set = "("
        for i in range(len(self.cam_folders)):
            file_set += selected_line[i + 1] + "|"
        file_set = file_set[:-1] + ")"
            
        command='mm3d SaisieAppuisPredicQt "{}" Ori-Bascule {} {}'.format(file_set,
                                                                   self.GCP_COORD_FILE_INIT,
                                                                   self.GCP_PICK_FILE)
        print(command)        
        utils.exec_mm3d(command)
        self.gcp_coord_file = opj(self.project_path,Photo4d.GCP_FOLDER, Photo4d.GCP_COORD_FILE_FINAL)
        
        # Go back from tmp dir to project dir        
        os.chdir(self.project_path)


    def process_all_timesteps(self, master_folder_id=0, clahe=False, tileGridSize_clahe=8, 
                              zoomF=1, Ori='Bascule', DefCor=0.0, shift=None, keep_rasters=True, display=False):
        if self.sorted_pictures is None:
            print("ERROR You must apply sort_pictures() before doing anything else")
            return

        proc.process_all_timesteps(self.tmp_path, self.sorted_pictures, opj(self.project_path, Photo4d.RESULT_FOLDER),
                          clahe=clahe, tileGridSize_clahe=tileGridSize_clahe, zoomF=zoomF,
                          master_folder_id=master_folder_id, Ori=Ori, DefCor=DefCor,
                          shift=shift, keep_rasters=keep_rasters, display_micmac=display)


    
    def set_selected_set(self, img_or_index: Union[int, str]):
        if self.sorted_pictures is None:
            print("ERROR You must apply sort_pictures before trying to chose a set")
            return
        else:
            if type(img_or_index) == int:
                self.selected_picture_set = img_or_index
                print(
                    "\n The current selected set is now {}".format(self.sorted_pictures[self.selected_picture_set][1:]))
            elif type(img_or_index) == str:
                found, i = False, 0
                while (not found) and (i < len(self.sorted_pictures)):
                    if img_or_index in self.sorted_pictures[i]:
                        found = True
                        self.selected_picture_set = i
                        print("\n The current selected set is now {}".format(
                            self.sorted_pictures[self.selected_picture_set][1:]))
                    i += 1
                if not found:
                    print('image {} not in sorted_pictures'.format(img_or_index))

        

    def clean_up_tmp(self):
        '''
        Function to delete the working folder.
        '''
        try:
            rmtree(self.tmp_path)  
        except FileNotFoundError:
            pass
        except PermissionError:
            print("Permission Denied, cannot delete " + self.tmp_path)
        except OSError:
            pass      
                
                
                
if __name__ == "__main__":
    
    # myproj = p4d.Photo4d(project_path=r"C:\Users\lucg\Desktop\Test_V1_2019")
    # myproj.sort_picture()
    # myproj.check_picture_quality()
    # myproj.timeSIFT_orientation()
    ## TODO : mask tie points
    # myproj.prepare_gcp_files(r"C:\Users\lucg\Desktop\Test_V1_2019\GCPs_coordinates_manual.txt",file_format="N_X_Y_Z")
    ## Select a set to input GCPs
    # myproj.set_selected_set("DSC02728.JPG")
    ## Input GCPs in 3 steps
    # myproj.pick_initial_gcps()
    # myproj.compute_transform()
    # myproj.pick_all_gcps()
    ## Eventually, change selected set to add GCP imput to more image (n times):
        #myproj.compute_transform()
        #myproj.set_selected_set("DSC02871.JPG")
        #myproj.pick_all_gcps()        
    #myproj.compute_transform(doCampari=True)
    
    ## FUNCTION TO CHANGE FOR TIMESIFT
    # myproj.create_mask()
    # myproj.process_all_timesteps()
    # myproj.clean_up_tmp()
    
    
    
    
    
    
    #OLD STUFF
    
    
    myproj = Photo4d(project_path=r"L:\Finse\Photo4D\Test_V1_2019")
   # myproj = Photo4d(project_path=r"~/icemassME/Finse/Photo4D/2018-1pm")
   # myproj.sort_picture()
    #myproj.check_picture_quality()
    
    #myproj.set_selected_set("DSC02728.JPG")
    #myproj.initial_orientation()
    #myproj.create_mask()
    #myproj.prepare_gcp_files(r"I:\icemass-users\lucg\Finse\Photo4D\2018-1pm\GCPs_coordinates_GNSS_Only.txt",file_format="N_X_Y_Z")
    #myproj.pick_initial_gcps()
    #myproj.compute_transform()
    #myproj.pick_all_gcps()
    #myproj.pick_gcp_final()
    #myproj.compute_transform(doCampari=True)
    

    
    
    #TODO: function to clean micmac diurectory (if needed)
    
    #myproj.detect_GCPs()
    #myproj.extract_GCPs()
    #myproj.graph_detected_gcps(50)
    #myproj.process(resol=4000)
    # Part 1, sort and flag good picture set
    # myproj.sort_picture()
    # myproj.check_picture_quality()

    # Part 2: Manual
    # choose either:
    # 		- Calibration, calibration of camera (TODO)
    # 		- Orientation, estimate camera position

    # myproj.selected_picture_set = -1  # by default get the last set of image. Change to the correct index where there are good quality images
    # myproj.oriententation_intitial()

    # myproj.selected_picture_set = -1
    # myproj.create_mask()

    # Part 3: Derive GCPS in image stack

    # myproj.prepare_GCP_files("GCPs.txt")  # format the text file to

    # myproj.selected_picture_set = -1
    # myproj.pick_GCP()

    # myproj.detect_GCPs()

    # point GCP (manual)
    # estimate on stack

    # Part 4: Process images to point cloud
    # myproj.process()
